@@ -18,29 +18,30 @@
  FOR A PARTICULAR PURPOSE.  See the license for more details.
 */
 
-#include <ql/experimental/callablebonds/callablebond.hpp>
-#include <ql/experimental/callablebonds/blackcallablebondengine.hpp>
 #include <ql/cashflows/cashflowvectors.hpp>
-#include <ql/termstructures/yield/zerospreadedtermstructure.hpp>
+#include <ql/experimental/callablebonds/blackcallablebondengine.hpp>
+#include <ql/experimental/callablebonds/callablebond.hpp>
 #include <ql/math/solvers1d/brent.hpp>
+#include <ql/termstructures/yield/zerospreadedtermstructure.hpp>
+#include <utility>
 
 namespace QuantLib {
 
     CallableBond::CallableBond(Natural settlementDays,
                                const Schedule& schedule,
-                               const DayCounter& paymentDayCounter,
+                               DayCounter paymentDayCounter,
                                const Date& issueDate,
-                               const CallabilitySchedule& putCallSchedule)
+                               CallabilitySchedule putCallSchedule)
     : Bond(settlementDays, schedule.calendar(), issueDate),
-      paymentDayCounter_(paymentDayCounter), putCallSchedule_(putCallSchedule) {
+      paymentDayCounter_(std::move(paymentDayCounter)),
+      putCallSchedule_(std::move(putCallSchedule)) {
 
         maturityDate_ = schedule.dates().back();
 
         if (!putCallSchedule_.empty()) {
             Date finalOptionDate = Date::minDate();
-            for (Size i=0; i<putCallSchedule_.size();++i) {
-                finalOptionDate=std::max(finalOptionDate,
-                                         putCallSchedule_[i]->date());
+            for (auto& i : putCallSchedule_) {
+                finalOptionDate = std::max(finalOptionDate, i->date());
             }
             QL_REQUIRE(finalOptionDate <= maturityDate_ ,
                        "Bond cannot mature before last call/put date");
@@ -126,7 +127,7 @@ namespace QuantLib {
 
     class OASHelper {
     public:
-        OASHelper(const boost::function<Real(Real)>& npvhelper,
+        OASHelper(const ext::function<Real(Real)>& npvhelper,
                   Real targetValue):
             npvhelper_(npvhelper),
             targetValue_(targetValue)
@@ -138,7 +139,7 @@ namespace QuantLib {
             return targetValue_ - npvhelper_(x);
         }
     private:
-        const boost::function<Real(Real)>& npvhelper_;
+        const ext::function<Real(Real)>& npvhelper_;
         Real targetValue_;
     };
 
@@ -153,10 +154,10 @@ namespace QuantLib {
                           Compounding compounding,
                           Frequency frequency)
     {
-        double zz=yts->zeroRate(b.maturityDate(),
-                                dayCounter,
-                                Continuous,
-                                NoFrequency);
+        Real zz=yts->zeroRate(b.maturityDate(),
+                              dayCounter,
+                              Continuous,
+                              NoFrequency);
         InterestRate baseRate(zz,
                               dayCounter,
                               Continuous,
@@ -165,16 +166,16 @@ namespace QuantLib {
                                   dayCounter,
                                   Continuous,
                                   NoFrequency);
-        double br=baseRate.equivalentRate(dayCounter,
-                                          compounding,
-                                          frequency,
-                                          yts->referenceDate(),
-                                          b.maturityDate()).rate();
-        double sr=spreadedRate.equivalentRate(dayCounter,
-                                              compounding,
-                                              frequency,
-                                              yts->referenceDate(),
-                                              b.maturityDate()).rate();
+        Real br=baseRate.equivalentRate(dayCounter,
+                                        compounding,
+                                        frequency,
+                                        yts->referenceDate(),
+                                        b.maturityDate()).rate();
+        Real sr=spreadedRate.equivalentRate(dayCounter,
+                                            compounding,
+                                            frequency,
+                                            yts->referenceDate(),
+                                            b.maturityDate()).rate();
         // Return the spread
         return sr-br;
     }
@@ -189,10 +190,10 @@ namespace QuantLib {
                           Compounding compounding,
                           Frequency frequency)
     {
-        double zz=yts->zeroRate(b.maturityDate(),
-                                dayCounter,
-                                compounding,
-                                frequency);
+        Real zz=yts->zeroRate(b.maturityDate(),
+                              dayCounter,
+                              compounding,
+                              frequency);
         InterestRate baseRate(zz,
                               dayCounter,
                               compounding,
@@ -202,16 +203,16 @@ namespace QuantLib {
                                   dayCounter,
                                   compounding,
                                   frequency);
-        double br=baseRate.equivalentRate(dayCounter,
-                                          Continuous,
-                                          NoFrequency,
-                                          yts->referenceDate(),
-                                          b.maturityDate()).rate();
-        double sr=spreadedRate.equivalentRate(dayCounter,
-                                              Continuous,
-                                              NoFrequency,
-                                              yts->referenceDate(),
-                                              b.maturityDate()).rate();
+        Real br=baseRate.equivalentRate(dayCounter,
+                                        Continuous,
+                                        NoFrequency,
+                                        yts->referenceDate(),
+                                        b.maturityDate()).rate();
+        Real sr=spreadedRate.equivalentRate(dayCounter,
+                                            Continuous,
+                                            NoFrequency,
+                                            yts->referenceDate(),
+                                            b.maturityDate()).rate();
         // Return the spread
         return sr-br;
     }
@@ -228,8 +229,7 @@ namespace QuantLib {
 
    Real CallableBond::NPVSpreadHelper::operator()(Real x) const
    {
-       CallableBond::arguments* args=
-           dynamic_cast<CallableBond::arguments*>(bond_.engine_->getArguments());
+       auto* args = dynamic_cast<CallableBond::arguments*>(bond_.engine_->getArguments());
        // Pops the original value when function finishes
        RestoreVal<Spread> restorer(args->spread);
        args->spread=x;
@@ -252,7 +252,7 @@ namespace QuantLib {
 
         Real dirtyPrice = cleanPrice + accruedAmount(settlement);
 
-        boost::function<Real(Real)> f = NPVSpreadHelper(*this);
+        ext::function<Real(Real)> f = NPVSpreadHelper(*this);
         OASHelper obj(f, dirtyPrice);
 
         Brent solver;
@@ -288,7 +288,7 @@ namespace QuantLib {
                              compounding,
                              frequency);
 
-        boost::function<Real(Real)> f = NPVSpreadHelper(*this);
+        ext::function<Real(Real)> f = NPVSpreadHelper(*this);
 
         Real P = f(oas) - accruedAmount(settlement);
 
@@ -370,7 +370,11 @@ namespace QuantLib {
                               BusinessDayConvention paymentConvention,
                               Real redemption,
                               const Date& issueDate,
-                              const CallabilitySchedule& putCallSchedule)
+                              const CallabilitySchedule& putCallSchedule,
+                              const Period& exCouponPeriod,
+                              const Calendar& exCouponCalendar,
+                              BusinessDayConvention exCouponConvention,
+                              bool exCouponEndOfMonth)
     : CallableBond(settlementDays, schedule, accrualDayCounter,
                    issueDate, putCallSchedule) {
 
@@ -383,7 +387,11 @@ namespace QuantLib {
                 FixedRateLeg(schedule)
                 .withNotionals(faceAmount)
                 .withCouponRates(coupons, accrualDayCounter)
-                .withPaymentAdjustment(paymentConvention);
+                .withPaymentAdjustment(paymentConvention)
+                .withExCouponPeriod(exCouponPeriod,
+                                    exCouponCalendar,
+                                    exCouponConvention,
+                                    exCouponEndOfMonth);
 
             addRedemptionsToCashflows(std::vector<Real>(1, redemption));
         } else {
@@ -406,12 +414,11 @@ namespace QuantLib {
         if (settlement == Date()) settlement = settlementDate();
 
         const bool IncludeToday = false;
-        for (Size i = 0; i<cashflows_.size(); ++i) {
+        for (const auto& cashflow : cashflows_) {
             // the first coupon paying after d is the one we're after
-            if (!cashflows_[i]->hasOccurred(settlement,IncludeToday)) {
-                ext::shared_ptr<Coupon> coupon =
-                    ext::dynamic_pointer_cast<Coupon>(cashflows_[i]);
-                if (coupon)
+            if (!cashflow->hasOccurred(settlement, IncludeToday)) {
+                ext::shared_ptr<Coupon> coupon = ext::dynamic_pointer_cast<Coupon>(cashflow);
+                if (coupon != nullptr)
                     // !!!
                     return coupon->accruedAmount(settlement) /
                            notional(settlement) * 100.0;
@@ -426,11 +433,11 @@ namespace QuantLib {
     void CallableFixedRateBond::setupArguments(
                                        PricingEngine::arguments* args) const {
 
-        Bond::setupArguments(args);
-        CallableBond::arguments* arguments =
-            dynamic_cast<CallableBond::arguments*>(args);
+        CallableBond::setupArguments(args);
 
-        QL_REQUIRE(arguments != 0, "no arguments given");
+        auto* arguments = dynamic_cast<CallableBond::arguments*>(args);
+
+        QL_REQUIRE(arguments != nullptr, "no arguments given");
 
         Date settlement = arguments->settlementDate;
 
@@ -445,7 +452,8 @@ namespace QuantLib {
         arguments->couponAmounts.reserve(cfs.size()-1);
 
         for (Size i=0; i<cfs.size()-1; i++) {
-            if (!cfs[i]->hasOccurred(settlement, false)) {
+            if (!cfs[i]->hasOccurred(settlement, false)
+                && !cfs[i]->tradingExCoupon(settlement)) {
                 arguments->couponDates.push_back(cfs[i]->date());
                 arguments->couponAmounts.push_back(cfs[i]->amount());
             }
@@ -460,22 +468,18 @@ namespace QuantLib {
         arguments->frequency = frequency_;
 
         arguments->putCallSchedule = putCallSchedule_;
-        for (Size i=0; i<putCallSchedule_.size(); i++) {
-            if (!putCallSchedule_[i]->hasOccurred(settlement, false)) {
-                arguments->callabilityDates.push_back(
-                                                 putCallSchedule_[i]->date());
-                arguments->callabilityPrices.push_back(
-                                       putCallSchedule_[i]->price().amount());
+        for (const auto& i : putCallSchedule_) {
+            if (!i->hasOccurred(settlement, false)) {
+                arguments->callabilityDates.push_back(i->date());
+                arguments->callabilityPrices.push_back(i->price().amount());
 
-                if (putCallSchedule_[i]->price().type()==
-                    Callability::Price::Clean) {
+                if (i->price().type() == Bond::Price::Clean) {
                     /* calling accrued() forces accrued interest to be zero
                        if future option date is also coupon date, so that dirty
                        price = clean price. Use here because callability is
                        always applied before coupon in the tree engine.
                     */
-                    arguments->callabilityPrices.back() +=
-                        this->accrued(putCallSchedule_[i]->date());
+                    arguments->callabilityPrices.back() += this->accrued(i->date());
                 }
             }
         }
