@@ -19,15 +19,11 @@
 
 #include <ql/experimental/catbonds/catrisk.hpp>
 #include <ql/time/daycounters/actualactual.hpp>
+#include <random>
 #include <utility>
 
 namespace QuantLib {
 
-    namespace {
-        Integer round(Real r) {
-            return (r > 0.0) ? Integer(std::floor(r + 0.5)) : Integer(std::ceil(r - 0.5));
-        }
-    }
 
     EventSetSimulation::EventSetSimulation(
         ext::shared_ptr<std::vector<std::pair<Date, Real> > > events,
@@ -84,36 +80,37 @@ namespace QuantLib {
     BetaRiskSimulation::BetaRiskSimulation(Date start, Date end, Real maxLoss, Real lambda, Real alpha, Real beta) 
               : CatSimulation(start, end), 
                 maxLoss_(maxLoss), 
-                exponential_(rng_, boost::exponential_distribution<>(lambda)),
-                gammaAlpha_(rng_, boost::gamma_distribution<>(alpha)),
-                gammaBeta_(rng_, boost::gamma_distribution<>(beta))
+                exponential_(lambda),
+                gammaAlpha_(alpha),
+                gammaBeta_(beta)
     {
-        ActualActual dayCounter;
+        DayCounter dayCounter = ActualActual(ActualActual::ISDA);
         dayCount_ = dayCounter.dayCount(start, end);
         yearFraction_ = dayCounter.yearFraction(start, end);
     }
 
     Real BetaRiskSimulation::generateBeta()
     {
-        Real X = gammaAlpha_();
-        Real Y = gammaBeta_();
+        Real X = gammaAlpha_(rng_);
+        Real Y = gammaBeta_(rng_);
         return X*maxLoss_/(X+Y);
     }
 
     bool BetaRiskSimulation::nextPath(std::vector<std::pair<Date, Real> > &path)
     {        
         path.resize(0);
-        Real eventFraction = exponential_();       
+        Real eventFraction = exponential_(rng_);
         while(eventFraction<=yearFraction_)
         {
-            Integer days = round(eventFraction*dayCount_/yearFraction_);
+            auto days =
+                static_cast<Integer>(std::lround(eventFraction * dayCount_ / yearFraction_));
             Date eventDate = start_ + days*Days;
             if(eventDate<=end_)
             {
                 path.emplace_back(eventDate, generateBeta());
             }
             else break;
-            eventFraction = exponential_();
+            eventFraction = exponential_(rng_);
         }
         return true;
     }
