@@ -30,8 +30,6 @@
 #include <ql/qldefines.hpp>
 #include <ql/math/matrixutilities/svd.hpp>
 #include <ql/math/array.hpp>
-#include <ql/math/functional.hpp>
-#include <boost/function.hpp>
 #include <boost/type_traits.hpp>
 #include <vector>
 
@@ -130,7 +128,7 @@ namespace QuantLib {
             if (w[i] > threshold) {
                 const Real u = std::inner_product(U.column_begin(i),
                     U.column_end(i),
-                    yBegin, 0.0)/w[i];
+                    yBegin, Real(0.0))/w[i];
 
                 for (Size j=0; j<m; ++j) {
                     a_[j]  +=u*V[j][i];
@@ -144,10 +142,10 @@ namespace QuantLib {
                        yBegin, residuals_.begin(), std::minus<Real>());
 
         const Real chiSq
-            = std::inner_product(residuals_.begin(), residuals_.end(),
-            residuals_.begin(), 0.0);
+            = std::inner_product(residuals_.begin(), residuals_.end(), residuals_.begin(), Real(0.0));
+        const Real multiplier = std::sqrt(chiSq/(n-2));
         std::transform(err_.begin(), err_.end(), standardErrors_.begin(),
-                       multiply_by<Real>(std::sqrt(chiSq/(n-2))));
+                       [=](Real x) -> Real { return x * multiplier; });
     }
 
 }
