@@ -40,6 +40,20 @@ namespace QuantLib {
         : public ZeroInflationTermStructure,
           protected InterpolatedCurve<Interpolator> {
       public:
+        InterpolatedZeroInflationCurve(const Date& referenceDate,
+                                       const Calendar& calendar,
+                                       const DayCounter& dayCounter,
+                                       const Period& lag,
+                                       Frequency frequency,
+                                       const std::vector<Date>& dates,
+                                       const std::vector<Rate>& rates,
+                                       const Interpolator& interpolator = Interpolator());
+
+        /*! \deprecated Use the constructor without the
+                        indexIsInterpolated parameter.
+                        Deprecated in version 1.25.
+        */
+        QL_DEPRECATED
         InterpolatedZeroInflationCurve(const Date& referenceDate,
                                        const Calendar& calendar,
                                        const DayCounter& dayCounter,
@@ -76,6 +90,19 @@ namespace QuantLib {
             (or can't) provide the points for interpolation on
             construction.
         */
+        InterpolatedZeroInflationCurve(const Date& referenceDate,
+                                       const Calendar& calendar,
+                                       const DayCounter& dayCounter,
+                                       const Period& lag,
+                                       Frequency frequency,
+                                       Rate baseZeroRate,
+                                       const Interpolator &interpolator = Interpolator());
+
+        /*! \deprecated Use the constructor without the
+                        indexIsInterpolated parameter.
+                        Deprecated in version 1.25.
+        */
+        QL_DEPRECATED
         InterpolatedZeroInflationCurve(const Date& referenceDate,
                                        const Calendar& calendar,
                                        const DayCounter& dayCounter,
@@ -83,8 +110,7 @@ namespace QuantLib {
                                        Frequency frequency,
                                        bool indexIsInterpolated,
                                        Rate baseZeroRate,
-                                       const Interpolator &interpolator
-                                                            = Interpolator());
+                                       const Interpolator &interpolator = Interpolator());
     };
 
     typedef InterpolatedZeroInflationCurve<Linear> ZeroInflationCurve;
@@ -93,6 +119,21 @@ namespace QuantLib {
 
     // template definitions
 
+    QL_DEPRECATED_DISABLE_WARNING
+
+    template <class Interpolator>
+    InterpolatedZeroInflationCurve<Interpolator>::InterpolatedZeroInflationCurve(
+        const Date& referenceDate,
+        const Calendar& calendar,
+        const DayCounter& dayCounter,
+        const Period& lag,
+        Frequency frequency,
+        const std::vector<Date>& dates,
+        const std::vector<Rate>& rates,
+        const Interpolator& interpolator)
+    : InterpolatedZeroInflationCurve(
+          referenceDate, calendar, dayCounter, lag, frequency, false, dates, rates, interpolator) {}
+
     template <class Interpolator>
     InterpolatedZeroInflationCurve<Interpolator>::InterpolatedZeroInflationCurve(
         const Date& referenceDate,
@@ -120,15 +161,6 @@ namespace QuantLib {
                    "first data date is not in base period, date: "
                        << dates_[0] << " not within [" << lim.first << "," << lim.second << "]");
 
-        // by convention, if the index is not interpolated we pull all the dates
-        // back to the start of their inflationPeriods
-        // otherwise the time calculations will be inconsistent
-        if (!indexIsInterpolated_) {
-            for (auto& date : dates_) {
-                date = inflationPeriod(date, frequency).first;
-            }
-        }
-
         QL_REQUIRE(this->data_.size() == dates_.size(),
                    "indices/dates count mismatch: " << this->data_.size() << " vs "
                                                     << dates_.size());
@@ -153,6 +185,20 @@ namespace QuantLib {
         this->interpolation_.update();
     }
 
+    template <class Interpolator>
+    InterpolatedZeroInflationCurve<Interpolator>::
+    InterpolatedZeroInflationCurve(const Date& referenceDate,
+                                   const Calendar& calendar,
+                                   const DayCounter& dayCounter,
+                                   const Period& lag,
+                                   Frequency frequency,
+                                   Rate baseZeroRate,
+                                   const Interpolator& interpolator)
+    :  ZeroInflationTermStructure(referenceDate, calendar, dayCounter,
+                                  baseZeroRate, lag, frequency),
+       InterpolatedCurve<Interpolator>(interpolator) {
+    }
+
     template <class Interpolator>
     InterpolatedZeroInflationCurve<Interpolator>::
     InterpolatedZeroInflationCurve(const Date& referenceDate,
@@ -168,25 +214,18 @@ namespace QuantLib {
        InterpolatedCurve<Interpolator>(interpolator) {
     }
 
+    QL_DEPRECATED_ENABLE_WARNING
 
     template <class T>
     Date InterpolatedZeroInflationCurve<T>::baseDate() const {
-        // if indexIsInterpolated we fixed the dates in the constructor
         return dates_.front();
     }
 
     template <class T>
     Date InterpolatedZeroInflationCurve<T>::maxDate() const {
-        Date d;
-        if (indexIsInterpolated()) {
-            d = dates_.back();
-        } else {
-            d = inflationPeriod(dates_.back(), frequency()).second;
-        }
-        return d;
+        return inflationPeriod(dates_.back(), frequency()).second;
     }
 
-
     template <class T>
     inline Rate InterpolatedZeroInflationCurve<T>::zeroRateImpl(Time t) const {
         return this->interpolation_(t, true);
