@@ -675,8 +675,8 @@ void CreditDefaultSwapTest::testIsdaEngine() {
     size_t l = 0;
 
     for (auto termDate : termDates) {
-        for (double spread : spreads) {
-            for (double& recoverie : recoveries) {
+        for (Real spread : spreads) {
+            for (Real& recoverie : recoveries) {
 
                 ext::shared_ptr<CreditDefaultSwap> quotedTrade =
                     MakeCreditDefaultSwap(termDate, spread).withNominal(10000000.);
@@ -696,7 +696,7 @@ void CreditDefaultSwapTest::testIsdaEngine() {
                         .withNominal(10000000.)
                         .withPricingEngine(engine);
 
-                BOOST_CHECK_CLOSE(conventionalTrade->notional() * conventionalTrade->fairUpfront(),
+                QL_CHECK_CLOSE(conventionalTrade->notional() * conventionalTrade->fairUpfront(),
                                   markitValues[l], tolerance);
 
                 l++;
@@ -738,7 +738,7 @@ void CreditDefaultSwapTest::testAccrualRebateAmounts() {
         Settings::instance().evaluationDate() = input.first;
         CreditDefaultSwap cds = MakeCreditDefaultSwap(maturity, spread)
             .withNominal(notional);
-        BOOST_CHECK_SMALL(input.second - cds.accrualRebate()->amount(), 0.01);
+        QL_CHECK_SMALL(input.second - cds.accrualRebate()->amount(), 0.01);
     }
 }
 
