@@ -21,6 +21,7 @@
 #include "fdheston.hpp"
 #include "utilities.hpp"
 
+#include <ql/math/functional.hpp>
 #include <ql/quotes/simplequote.hpp>
 #include <ql/time/calendars/target.hpp>
 #include <ql/time/daycounters/actual360.hpp>
@@ -31,6 +32,7 @@
 #include <ql/models/equity/hestonmodel.hpp>
 #include <ql/termstructures/yield/zerocurve.hpp>
 #include <ql/termstructures/yield/flatforward.hpp>
+#include <ql/termstructures/volatility/equityfx/localconstantvol.hpp>
 #include <ql/methods/finitedifferences/meshers/fdmhestonvariancemesher.hpp>
 #include <ql/pricingengines/barrier/analyticbarrierengine.hpp>
 #include <ql/pricingengines/vanilla/analytichestonengine.hpp>
@@ -39,17 +41,13 @@
 #include <ql/pricingengines/vanilla/fdhestonvanillaengine.hpp>
 #include <ql/pricingengines/barrier/fdblackscholesbarrierengine.hpp>
 #include <ql/pricingengines/vanilla/fdblackscholesvanillaengine.hpp>
-
-#include <boost/make_shared.hpp>
-#include <boost/assign/std/vector.hpp>
-#include <boost/make_shared.hpp>
+#include <ql/tuple.hpp>
 
 using namespace QuantLib;
-using namespace boost::assign;
 using boost::unit_test_framework::test_suite;
 
 
-namespace {
+namespace fd_heston_test {
     struct NewBarrierOptionData {
         Barrier::Type barrierType;
         Real barrier;
@@ -62,26 +60,57 @@ namespace {
         Time t;        // time to maturity
         Volatility v;  // volatility
     };
+
+    class ParableLocalVolatility : public LocalVolTermStructure {
+      public:
+        ParableLocalVolatility(
+            const Date& referenceDate,
+            Real s0,
+            Real alpha,
+            const DayCounter& dayCounter)
+        : LocalVolTermStructure(
+              referenceDate, NullCalendar(), Following, dayCounter),
+          referenceDate_(referenceDate),
+          s0_(s0),
+          alpha_(alpha) {}
+
+        Date maxDate() const override { return Date::maxDate(); }
+        Real minStrike() const override { return 0.0; }
+        Real maxStrike() const override { return std::numeric_limits<Real>::max(); }
+
+      protected:
+        Volatility localVolImpl(Time t, Real s) const override {
+            return alpha_*(squared(s0_ - s) + 25.0);
+        }
+
+      private:
+        const Date referenceDate_;
+        const Real s0_, alpha_;
+    };
 }
 
 void FdHestonTest::testFdmHestonVarianceMesher() {
     BOOST_TEST_MESSAGE("Testing FDM Heston variance mesher...");
 
+    using namespace fd_heston_test;
+
     SavedSettings backup;
 
     const Date today = Date(22, February, 2018);
     const DayCounter dc = Actual365Fixed();
     Settings::instance().evaluationDate() = today;
 
-    const boost::shared_ptr<HestonProcess> process(
-        boost::make_shared<HestonProcess>(
+    const ext::shared_ptr<HestonProcess> process(
+        ext::make_shared<HestonProcess>(
             Handle<YieldTermStructure>(flatRate(0.02, dc)),
             Handle<YieldTermStructure>(flatRate(0.02, dc)),
-            Handle<Quote>(boost::make_shared<SimpleQuote>(100.0)),
+            Handle<Quote>(ext::make_shared<SimpleQuote>(100.0)),
             0.09, 1.0, 0.09, 0.2, -0.5));
 
-    const std::vector<Real> locations =
-        FdmHestonVarianceMesher(5, process, 1.0).locations();
+    const ext::shared_ptr<FdmHestonVarianceMesher> mesher
+        = ext::make_shared<FdmHestonVarianceMesher>(5, process, 1.0);
+
+    const std::vector<Real> locations = mesher->locations();
 
     const Real expected[] = {
         0.0, 6.652314e-02, 9.000000e-02, 1.095781e-01, 2.563610e-01
@@ -100,12 +129,72 @@ void FdHestonTest::testFdmHestonVarianceMesher() {
                         << "\n    tolerance:  " << tol);
         }
     }
+
+    const ext::shared_ptr<LocalVolTermStructure> lVol =
+        ext::make_shared<LocalConstantVol>(today, 2.5, dc);
+
+    const ext::shared_ptr<FdmHestonLocalVolatilityVarianceMesher> constSlvMesher
+        = ext::make_shared<FdmHestonLocalVolatilityVarianceMesher>
+              (5, process, lVol, 1.0);
+
+    const Real expectedVol = 2.5 * mesher->volaEstimate();
+    const Real calculatedVol = constSlvMesher->volaEstimate();
+
+    const Real diff = std::fabs(calculatedVol - expectedVol);
+    if (diff > tol) {
+        BOOST_ERROR("Failed to reproduce Heston local volatility "
+                "variance estimate"
+                    << "\n    calculated: " << calculatedVol
+                    << "\n    expected:   " << expectedVol
+                    << std::scientific
+                    << "\n    difference  " << diff
+                    << "\n    tolerance:  " << tol);
+    }
+
+    const Real alpha = 0.01;
+    const ext::shared_ptr<LocalVolTermStructure> leverageFct
+        = ext::make_shared<ParableLocalVolatility>(today, 100.0, alpha, dc);
+
+    const ext::shared_ptr<FdmHestonLocalVolatilityVarianceMesher> slvMesher
+        = ext::make_shared<FdmHestonLocalVolatilityVarianceMesher>(
+              5, process, leverageFct, 0.5, 1, 0.01);
+
+    const Real initialVolEstimate =
+        ext::make_shared<FdmHestonVarianceMesher>(5, process, 0.5, 1, 0.01)->
+            volaEstimate();
+
+    // const Real vEst = leverageFct->localVol(0, 100) * initialVolEstimate;
+    // Mathematica solution
+    //    N[Integrate[
+    //      alpha*((100*Exp[vEst*x*Sqrt[0.5]] - 100)^2 + 25)*
+    //       PDF[NormalDistribution[0, 1], x], {x ,
+    //       InverseCDF[NormalDistribution[0, 1], 0.01],
+    //       InverseCDF[NormalDistribution[0, 1], 0.99]}]]
+
+    const Real leverageAvg = 0.455881 / (1-0.02);
+
+    const Real volaEstExpected =
+        0.5*(leverageAvg + leverageFct->localVol(0, 100)) * initialVolEstimate;
+
+    const Real volaEstCalculated = slvMesher->volaEstimate();
+
+    if (std::fabs(volaEstExpected - volaEstCalculated) > 0.001) {
+        BOOST_ERROR("Failed to reproduce Heston local volatility "
+                "variance estimate"
+                    << "\n    calculated: " << calculatedVol
+                    << "\n    expected:   " << expectedVol
+                    << std::scientific
+                    << "\n    difference  " << std::fabs(volaEstExpected - volaEstCalculated)
+                    << "\n    tolerance:  " << tol);
+    }
 }
 
 void FdHestonTest::testFdmHestonBarrierVsBlackScholes() {
 
     BOOST_TEST_MESSAGE("Testing FDM with barrier option in Heston model...");
 
+    using namespace fd_heston_test;
+
     SavedSettings backup;
 
     NewBarrierOptionData values[] = {
@@ -200,43 +289,41 @@ void FdHestonTest::testFdmHestonBarrierVsBlackScholes() {
     Settings::instance().evaluationDate() = todaysDate;
 
     Handle<Quote> spot(
-            boost::shared_ptr<Quote>(new SimpleQuote(0.0)));
-    boost::shared_ptr<SimpleQuote> qRate(new SimpleQuote(0.0));
+            ext::shared_ptr<Quote>(new SimpleQuote(0.0)));
+    ext::shared_ptr<SimpleQuote> qRate(new SimpleQuote(0.0));
     Handle<YieldTermStructure> qTS(flatRate(qRate, dc));
-    boost::shared_ptr<SimpleQuote> rRate(new SimpleQuote(0.0));
+    ext::shared_ptr<SimpleQuote> rRate(new SimpleQuote(0.0));
     Handle<YieldTermStructure> rTS(flatRate(rRate, dc));
-    boost::shared_ptr<SimpleQuote> vol(new SimpleQuote(0.0));
+    ext::shared_ptr<SimpleQuote> vol(new SimpleQuote(0.0));
     Handle<BlackVolTermStructure> volTS(flatVol(vol, dc));
 
-    boost::shared_ptr<BlackScholesMertonProcess> bsProcess(
+    ext::shared_ptr<BlackScholesMertonProcess> bsProcess(
                       new BlackScholesMertonProcess(spot, qTS, rTS, volTS));
 
-    boost::shared_ptr<PricingEngine> analyticEngine(
+    ext::shared_ptr<PricingEngine> analyticEngine(
                                         new AnalyticBarrierEngine(bsProcess));
-    
-    for (Size i=0; i<LENGTH(values); i++) {
-        Date exDate = todaysDate + Integer(values[i].t*365+0.5);
-        boost::shared_ptr<Exercise> exercise(new EuropeanExercise(exDate));
 
-        boost::dynamic_pointer_cast<SimpleQuote>(spot .currentLink())
-                                                    ->setValue(values[i].s);
-        qRate->setValue(values[i].q);
-        rRate->setValue(values[i].r);
-        vol  ->setValue(values[i].v);
+    for (auto& value : values) {
+        Date exDate = todaysDate + timeToDays(value.t, 365);
+        ext::shared_ptr<Exercise> exercise(new EuropeanExercise(exDate));
 
-        boost::shared_ptr<StrikedTypePayoff> payoff(new
-                    PlainVanillaPayoff(values[i].type, values[i].strike));
+        ext::dynamic_pointer_cast<SimpleQuote>(spot.currentLink())->setValue(value.s);
+        qRate->setValue(value.q);
+        rRate->setValue(value.r);
+        vol->setValue(value.v);
 
-        BarrierOption barrierOption(values[i].barrierType, values[i].barrier,
-                                    values[i].rebate, payoff, exercise);
+        ext::shared_ptr<StrikedTypePayoff> payoff(new PlainVanillaPayoff(value.type, value.strike));
+
+        BarrierOption barrierOption(value.barrierType, value.barrier, value.rebate, payoff,
+                                    exercise);
 
         const Real v0 = vol->value()*vol->value();
-        boost::shared_ptr<HestonProcess> hestonProcess(
+        ext::shared_ptr<HestonProcess> hestonProcess(
              new HestonProcess(rTS, qTS, spot, v0, 1.0, v0, 0.005, 0.0));
 
-        barrierOption.setPricingEngine(boost::shared_ptr<PricingEngine>(
-            new FdHestonBarrierEngine(boost::shared_ptr<HestonModel>(
-                              new HestonModel(hestonProcess)), 200, 101, 3)));
+        barrierOption.setPricingEngine(ext::shared_ptr<PricingEngine>(
+            new FdHestonBarrierEngine(ext::make_shared<HestonModel>(
+                              hestonProcess), 200, 101, 3)));
 
         const Real calculatedHE = barrierOption.NPV();
     
@@ -260,27 +347,27 @@ void FdHestonTest::testFdmHestonBarrier() {
 
     SavedSettings backup;
 
-    Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(100.0)));
+    Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(100.0)));
 
     Handle<YieldTermStructure> rTS(flatRate(0.05, Actual365Fixed()));
     Handle<YieldTermStructure> qTS(flatRate(0.0 , Actual365Fixed()));
 
-    boost::shared_ptr<HestonProcess> hestonProcess(
+    ext::shared_ptr<HestonProcess> hestonProcess(
         new HestonProcess(rTS, qTS, s0, 0.04, 2.5, 0.04, 0.66, -0.8));
 
     Settings::instance().evaluationDate() = Date(28, March, 2004);
     Date exerciseDate(28, March, 2005);
 
-    boost::shared_ptr<Exercise> exercise(new EuropeanExercise(exerciseDate));
+    ext::shared_ptr<Exercise> exercise(new EuropeanExercise(exerciseDate));
 
-    boost::shared_ptr<StrikedTypePayoff> payoff(new
+    ext::shared_ptr<StrikedTypePayoff> payoff(new
                                       PlainVanillaPayoff(Option::Call, 100));
 
     BarrierOption barrierOption(Barrier::UpOut, 135, 0.0, payoff, exercise);
 
-    barrierOption.setPricingEngine(boost::shared_ptr<PricingEngine>(
-            new FdHestonBarrierEngine(boost::shared_ptr<HestonModel>(
-                              new HestonModel(hestonProcess)), 50, 400, 100)));
+    barrierOption.setPricingEngine(ext::shared_ptr<PricingEngine>(
+            new FdHestonBarrierEngine(ext::make_shared<HestonModel>(
+                              hestonProcess), 50, 400, 100)));
 
     const Real tol = 0.01;
     const Real npvExpected   =  9.1530;
@@ -313,26 +400,26 @@ void FdHestonTest::testFdmHestonAmerican() {
 
     SavedSettings backup;
 
-    Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(100.0)));
+    Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(100.0)));
 
     Handle<YieldTermStructure> rTS(flatRate(0.05, Actual365Fixed()));
     Handle<YieldTermStructure> qTS(flatRate(0.0 , Actual365Fixed()));
 
-    boost::shared_ptr<HestonProcess> hestonProcess(
+    ext::shared_ptr<HestonProcess> hestonProcess(
         new HestonProcess(rTS, qTS, s0, 0.04, 2.5, 0.04, 0.66, -0.8));
 
     Settings::instance().evaluationDate() = Date(28, March, 2004);
     Date exerciseDate(28, March, 2005);
 
-    boost::shared_ptr<Exercise> exercise(new AmericanExercise(exerciseDate));
+    ext::shared_ptr<Exercise> exercise(new AmericanExercise(exerciseDate));
 
-    boost::shared_ptr<StrikedTypePayoff> payoff(new
+    ext::shared_ptr<StrikedTypePayoff> payoff(new
                                       PlainVanillaPayoff(Option::Put, 100));
 
     VanillaOption option(payoff, exercise);
-    boost::shared_ptr<PricingEngine> engine(
-         new FdHestonVanillaEngine(boost::shared_ptr<HestonModel>(
-                             new HestonModel(hestonProcess)), 200, 100, 50));
+    ext::shared_ptr<PricingEngine> engine(
+         new FdHestonVanillaEngine(ext::make_shared<HestonModel>(
+                             hestonProcess), 200, 100, 50));
     option.setPricingEngine(engine);
     
     const Real tol = 0.01;
@@ -378,9 +465,9 @@ void FdHestonTest::testFdmHestonIkonenToivanen() {
     Settings::instance().evaluationDate() = Date(28, March, 2004);
     Date exerciseDate(26, June, 2004);
 
-    boost::shared_ptr<Exercise> exercise(new AmericanExercise(exerciseDate));
+    ext::shared_ptr<Exercise> exercise(new AmericanExercise(exerciseDate));
 
-    boost::shared_ptr<StrikedTypePayoff> payoff(new
+    ext::shared_ptr<StrikedTypePayoff> payoff(new
                                       PlainVanillaPayoff(Option::Put, 10));
 
     VanillaOption option(payoff, exercise);
@@ -390,13 +477,13 @@ void FdHestonTest::testFdmHestonIkonenToivanen() {
     const Real tol = 0.001;
     
     for (Size i=0; i < LENGTH(strikes); ++i) {
-        Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(strikes[i])));
-        boost::shared_ptr<HestonProcess> hestonProcess(
+        Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(strikes[i])));
+        ext::shared_ptr<HestonProcess> hestonProcess(
             new HestonProcess(rTS, qTS, s0, 0.0625, 5, 0.16, 0.9, 0.1));
     
-        boost::shared_ptr<PricingEngine> engine(
-             new FdHestonVanillaEngine(boost::shared_ptr<HestonModel>(
-                                 new HestonModel(hestonProcess)), 100, 400));
+        ext::shared_ptr<PricingEngine> engine(
+             new FdHestonVanillaEngine(ext::make_shared<HestonModel>(
+                                 hestonProcess), 100, 400));
         option.setPricingEngine(engine);
         
         Real calculated = option.NPV();
@@ -425,59 +512,55 @@ void FdHestonTest::testFdmHestonBlackScholes() {
     Handle<BlackVolTermStructure> volTS(
                     flatVol(rTS->referenceDate(), 0.25, rTS->dayCounter()));
     
-    boost::shared_ptr<Exercise> exercise(new EuropeanExercise(exerciseDate));
+    ext::shared_ptr<Exercise> exercise(new EuropeanExercise(exerciseDate));
 
-    boost::shared_ptr<StrikedTypePayoff> payoff(new
+    ext::shared_ptr<StrikedTypePayoff> payoff(new
                                       PlainVanillaPayoff(Option::Put, 10));
 
     VanillaOption option(payoff, exercise);
 
     Real strikes[]  = { 8, 9, 10, 11, 12 };
     const Real tol = 0.0001;
-    
-    for (Size i=0; i < LENGTH(strikes); ++i) {
-        Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(strikes[i])));
 
-        boost::shared_ptr<GeneralizedBlackScholesProcess> bsProcess(
+    for (Real& strike : strikes) {
+        Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(strike)));
+
+        ext::shared_ptr<GeneralizedBlackScholesProcess> bsProcess(
                        new GeneralizedBlackScholesProcess(s0, qTS, rTS, volTS));
 
-        option.setPricingEngine(boost::shared_ptr<PricingEngine>(
+        option.setPricingEngine(ext::shared_ptr<PricingEngine>(
                                         new AnalyticEuropeanEngine(bsProcess)));
         
         const Real expected = option.NPV();
         
-        boost::shared_ptr<HestonProcess> hestonProcess(
+        ext::shared_ptr<HestonProcess> hestonProcess(
             new HestonProcess(rTS, qTS, s0, 0.0625, 1, 0.0625, 0.0001, 0.0));
 
         // Hundsdorfer scheme
-        option.setPricingEngine(boost::shared_ptr<PricingEngine>(
-             new FdHestonVanillaEngine(boost::shared_ptr<HestonModel>(
-                                           new HestonModel(hestonProcess)), 
+        option.setPricingEngine(ext::shared_ptr<PricingEngine>(
+             new FdHestonVanillaEngine(ext::make_shared<HestonModel>(
+                                           hestonProcess), 
                                        100, 400, 3)));
         
         Real calculated = option.NPV();
         if (std::fabs(calculated - expected) > tol) {
             BOOST_ERROR("Failed to reproduce expected npv"
-                        << "\n    strike:     " << strikes[i]
-                        << "\n    calculated: " << calculated
-                        << "\n    expected:   " << expected
-                        << "\n    tolerance:  " << tol); 
+                        << "\n    strike:     " << strike << "\n    calculated: " << calculated
+                        << "\n    expected:   " << expected << "\n    tolerance:  " << tol);
         }
         
         // Explicit scheme
-        option.setPricingEngine(boost::shared_ptr<PricingEngine>(
-             new FdHestonVanillaEngine(boost::shared_ptr<HestonModel>(
-                                           new HestonModel(hestonProcess)),
-                                       500, 400, 3, 0,
+        option.setPricingEngine(ext::shared_ptr<PricingEngine>(
+             new FdHestonVanillaEngine(ext::make_shared<HestonModel>(
+                                           hestonProcess),
+                                       4000, 400, 3, 0,
                                        FdmSchemeDesc::ExplicitEuler())));
 
         calculated = option.NPV();
         if (std::fabs(calculated - expected) > tol) {
             BOOST_ERROR("Failed to reproduce expected npv"
-                        << "\n    strike:     " << strikes[i]
-                        << "\n    calculated: " << calculated
-                        << "\n    expected:   " << expected
-                        << "\n    tolerance:  " << tol);
+                        << "\n    strike:     " << strike << "\n    calculated: " << calculated
+                        << "\n    expected:   " << expected << "\n    tolerance:  " << tol);
         }
     }
 }
@@ -491,36 +574,36 @@ void FdHestonTest::testFdmHestonEuropeanWithDividends() {
 
     SavedSettings backup;
 
-    Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(100.0)));
+    Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(100.0)));
 
     Handle<YieldTermStructure> rTS(flatRate(0.05, Actual365Fixed()));
     Handle<YieldTermStructure> qTS(flatRate(0.0 , Actual365Fixed()));
 
-    boost::shared_ptr<HestonProcess> hestonProcess(
+    ext::shared_ptr<HestonProcess> hestonProcess(
         new HestonProcess(rTS, qTS, s0, 0.04, 2.5, 0.04, 0.66, -0.8));
 
     Settings::instance().evaluationDate() = Date(28, March, 2004);
     Date exerciseDate(28, March, 2005);
 
-    boost::shared_ptr<Exercise> exercise(new AmericanExercise(exerciseDate));
+    ext::shared_ptr<Exercise> exercise(new AmericanExercise(exerciseDate));
 
-    boost::shared_ptr<StrikedTypePayoff> payoff(new
+    ext::shared_ptr<StrikedTypePayoff> payoff(new
                                       PlainVanillaPayoff(Option::Put, 100));
 
     const std::vector<Real> dividends(1, 5);
     const std::vector<Date> dividendDates(1, Date(28, September, 2004));
 
     DividendVanillaOption option(payoff, exercise, dividendDates, dividends);
-    boost::shared_ptr<PricingEngine> engine(
-         new FdHestonVanillaEngine(boost::shared_ptr<HestonModel>(
-                             new HestonModel(hestonProcess)), 50, 100, 50));
+    ext::shared_ptr<PricingEngine> engine(
+         new FdHestonVanillaEngine(ext::make_shared<HestonModel>(
+                             hestonProcess), 50, 100, 50));
     option.setPricingEngine(engine);
     
     const Real tol = 0.01;
     const Real gammaTol = 0.001;
-    const Real npvExpected   =  7.365075;
-    const Real deltaExpected = -0.396678;
-    const Real gammaExpected =  0.027681;
+    const Real npvExpected   =  7.38216;
+    const Real deltaExpected = -0.397902;
+    const Real gammaExpected =  0.027747;
         
     if (std::fabs(option.NPV() - npvExpected) > tol) {
         BOOST_ERROR("Failed to reproduce expected npv"
@@ -573,59 +656,52 @@ void FdHestonTest::testFdmHestonConvergence() {
         { 2.5   , 0.06  , 0.5   , -0.1   , 0.0507, 0.0469, 0.25, 100 }
     };
 
-    FdmSchemeDesc schemes[] = { FdmSchemeDesc::Hundsdorfer(), 
-                                FdmSchemeDesc::ModifiedCraigSneyd(),
-                                FdmSchemeDesc::ModifiedHundsdorfer(), 
-                                FdmSchemeDesc::CraigSneyd() };
+    FdmSchemeDesc schemes[] = {
+        FdmSchemeDesc::Hundsdorfer(),
+        FdmSchemeDesc::ModifiedCraigSneyd(),
+        FdmSchemeDesc::ModifiedHundsdorfer(),
+        FdmSchemeDesc::CraigSneyd(),
+        FdmSchemeDesc::TrBDF2(),
+        FdmSchemeDesc::CrankNicolson(),
+    };
     
-    Size tn[] = { 100 };
+    Size tn[] = { 60 };
     Real v0[] = { 0.04 };
     
     const Date todaysDate(28, March, 2004); 
     Settings::instance().evaluationDate() = todaysDate;
     
-    Handle<Quote> s0(boost::shared_ptr<Quote>(new SimpleQuote(75.0)));
+    Handle<Quote> s0(ext::shared_ptr<Quote>(new SimpleQuote(75.0)));
 
-    for (Size l=0; l < LENGTH(schemes); ++l) {
-        for (Size i=0; i < LENGTH(values); ++i) {
-            for (Size j=0; j < LENGTH(tn); ++j) {
-                for (Size k=0; k < LENGTH(v0); ++k) {
-                    Handle<YieldTermStructure> rTS(
-                        flatRate(values[i].r, Actual365Fixed()));
-                    Handle<YieldTermStructure> qTS(
-                        flatRate(values[i].q, Actual365Fixed()));
-                
-                    boost::shared_ptr<HestonProcess> hestonProcess(
-                        new HestonProcess(rTS, qTS, s0, 
-                                          v0[k], 
-                                          values[i].kappa, 
-                                          values[i].theta, 
-                                          values[i].sigma, 
-                                          values[i].rho));
-                
-                    Date exerciseDate = todaysDate 
-                        + Period(static_cast<Integer>(values[i].T*365), Days);
-                    boost::shared_ptr<Exercise> exercise(
+    for (const auto& scheme : schemes) {
+        for (auto& value : values) {
+            for (unsigned long j : tn) {
+                for (Real k : v0) {
+                    Handle<YieldTermStructure> rTS(flatRate(value.r, Actual365Fixed()));
+                    Handle<YieldTermStructure> qTS(flatRate(value.q, Actual365Fixed()));
+
+                    ext::shared_ptr<HestonProcess> hestonProcess(new HestonProcess(
+                        rTS, qTS, s0, k, value.kappa, value.theta, value.sigma, value.rho));
+
+                    Date exerciseDate =
+                        todaysDate + Period(static_cast<Integer>(value.T * 365), Days);
+                    ext::shared_ptr<Exercise> exercise(
                                           new EuropeanExercise(exerciseDate));
-                
-                    boost::shared_ptr<StrikedTypePayoff> payoff(new
-                               PlainVanillaPayoff(Option::Call, values[i].K));
-            
+
+                    ext::shared_ptr<StrikedTypePayoff> payoff(
+                        new PlainVanillaPayoff(Option::Call, value.K));
+
                     VanillaOption option(payoff, exercise);
-                    boost::shared_ptr<PricingEngine> engine(
-                         new FdHestonVanillaEngine(
-                             boost::shared_ptr<HestonModel>(
-                                 new HestonModel(hestonProcess)), 
-                             tn[j], 101, 51, 0,
-                             schemes[l]));
+                    ext::shared_ptr<PricingEngine> engine(new FdHestonVanillaEngine(
+                        ext::make_shared<HestonModel>(hestonProcess), j, 101, 51, 0, scheme));
                     option.setPricingEngine(engine);
                     
                     const Real calculated = option.NPV();
                     
-                    boost::shared_ptr<PricingEngine> analyticEngine(
+                    ext::shared_ptr<PricingEngine> analyticEngine(
                         new AnalyticHestonEngine(
-                            boost::shared_ptr<HestonModel>(
-                                new HestonModel(hestonProcess)), 144));
+                            ext::make_shared<HestonModel>(
+                                hestonProcess), 144));
                     
                     option.setPricingEngine(analyticEngine);
                     const Real expected = option.NPV();
@@ -649,7 +725,6 @@ void FdHestonTest::testFdmHestonIntradayPricing() {
 
     SavedSettings backup;
 
-    const Calendar calendar = TARGET();
     const Option::Type type(Option::Put);
     const Real underlying = 36;
     const Real strike = underlying;
@@ -664,21 +739,21 @@ void FdHestonTest::testFdmHestonIntradayPricing() {
 
     const Date maturity(17, May, 2014, 17, 30, 0);
 
-    const boost::shared_ptr<Exercise> europeanExercise(
+    const ext::shared_ptr<Exercise> europeanExercise(
         new EuropeanExercise(maturity));
-    const boost::shared_ptr<StrikedTypePayoff> payoff(
+    const ext::shared_ptr<StrikedTypePayoff> payoff(
         new PlainVanillaPayoff(type, strike));
     VanillaOption option(payoff, europeanExercise);
 
     const Handle<Quote> s0(
-         boost::shared_ptr<Quote>(new SimpleQuote(underlying)));
+         ext::shared_ptr<Quote>(new SimpleQuote(underlying)));
     RelinkableHandle<BlackVolTermStructure> flatVolTS;
     RelinkableHandle<YieldTermStructure> flatTermStructure, flatDividendTS;
-    const boost::shared_ptr<HestonProcess> process(
+    const ext::shared_ptr<HestonProcess> process(
         new HestonProcess(flatTermStructure, flatDividendTS, s0,
               v0, kappa, theta, sigma, rho));
-    const boost::shared_ptr<HestonModel> model(new HestonModel(process));
-    const boost::shared_ptr<PricingEngine> fdm(
+    const ext::shared_ptr<HestonModel> model(new HestonModel(process));
+    const ext::shared_ptr<PricingEngine> fdm(
         new FdHestonVanillaEngine(model, 20, 100, 26, 0));
     option.setPricingEngine(fdm);
 
@@ -690,9 +765,9 @@ void FdHestonTest::testFdmHestonIntradayPricing() {
         const Date now(17, May, 2014, 15, i*15, 0);
         Settings::instance().evaluationDate() = now;
 
-        flatTermStructure.linkTo(boost::shared_ptr<YieldTermStructure>(
+        flatTermStructure.linkTo(ext::shared_ptr<YieldTermStructure>(
             new FlatForward(now, riskFreeRate, dayCounter)));
-        flatDividendTS.linkTo(boost::shared_ptr<YieldTermStructure>(
+        flatDividendTS.linkTo(ext::shared_ptr<YieldTermStructure>(
             new FlatForward(now, dividendYield, dayCounter)));
 
         const Real gammaCalculated = option.gamma();
@@ -707,7 +782,7 @@ void FdHestonTest::testFdmHestonIntradayPricing() {
 #endif
 }
 
-void FdHestonTest::testMethodOfLines() {
+void FdHestonTest::testMethodOfLinesAndCN() {
     BOOST_TEST_MESSAGE("Testing method of lines to solve Heston PDEs...");
 
     SavedSettings backup;
@@ -717,7 +792,7 @@ void FdHestonTest::testMethodOfLines() {
 
     Settings::instance().evaluationDate() = today;
 
-    const Handle<Quote> spot(boost::make_shared<SimpleQuote>(100.0));
+    const Handle<Quote> spot(ext::make_shared<SimpleQuote>(100.0));
     const Handle<YieldTermStructure> qTS(flatRate(today, 0.0, dc));
     const Handle<YieldTermStructure> rTS(flatRate(today, 0.0, dc));
 
@@ -729,74 +804,183 @@ void FdHestonTest::testMethodOfLines() {
 
     const Date maturity = today + Period(3, Months);
 
-    const boost::shared_ptr<HestonModel> model(
-        boost::make_shared<HestonModel>(
-            boost::make_shared<HestonProcess>(
+    const ext::shared_ptr<HestonModel> model(
+        ext::make_shared<HestonModel>(
+            ext::make_shared<HestonProcess>(
                 rTS, qTS, spot, v0, kappa, theta, sigma, rho)));
 
     const Size xGrid = 21;
     const Size vGrid = 7;
 
-    const boost::shared_ptr<PricingEngine> fdmDefault(
-        boost::make_shared<FdHestonVanillaEngine>(model, 10, xGrid, vGrid, 0));
+    const ext::shared_ptr<PricingEngine> fdmDefault(
+        ext::make_shared<FdHestonVanillaEngine>(model, 10, xGrid, vGrid, 0));
 
-    const boost::shared_ptr<PricingEngine> fdmMol(
-        boost::make_shared<FdHestonVanillaEngine>(
+    const ext::shared_ptr<PricingEngine> fdmMol(
+        ext::make_shared<FdHestonVanillaEngine>(
             model, 10, xGrid, vGrid, 0, FdmSchemeDesc::MethodOfLines()));
 
-    const boost::shared_ptr<PlainVanillaPayoff> payoff =
-        boost::make_shared<PlainVanillaPayoff>(Option::Put, spot->value());
+    const ext::shared_ptr<PlainVanillaPayoff> payoff =
+        ext::make_shared<PlainVanillaPayoff>(Option::Put, spot->value());
 
     VanillaOption option(
-        payoff, boost::make_shared<AmericanExercise>(maturity));
+        payoff, ext::make_shared<AmericanExercise>(maturity));
 
     option.setPricingEngine(fdmMol);
-    const Real calculated = option.NPV();
+    const Real calculatedMoL = option.NPV();
 
     option.setPricingEngine(fdmDefault);
     const Real expected = option.NPV();
 
     const Real tol = 0.005;
-    const Real diff = std::fabs(expected - calculated);
+    const Real diffMoL = std::fabs(expected - calculatedMoL);
 
-    if (diff > tol) {
+    if (diffMoL > tol) {
         BOOST_FAIL("Failed to reproduce european option values with MOL"
-                   << "\n    calculated: " << calculated
+                   << "\n    calculated: " << calculatedMoL
                    << "\n    expected:   " << expected
-                   << "\n    difference: " << diff
+                   << "\n    difference: " << diffMoL
+                   << "\n    tolerance:  " << tol);
+    }
+
+    const ext::shared_ptr<PricingEngine> fdmCN(
+        ext::make_shared<FdHestonVanillaEngine>(
+            model, 10, xGrid, vGrid, 0, FdmSchemeDesc::CrankNicolson()));
+    option.setPricingEngine(fdmCN);
+
+    const Real calculatedCN = option.NPV();
+    const Real diffCN = std::fabs(expected - calculatedCN);
+
+    if (diffCN > tol) {
+        BOOST_FAIL("Failed to reproduce european option values with Crank-Nicolson"
+                   << "\n    calculated: " << calculatedCN
+                   << "\n    expected:   " << expected
+                   << "\n    difference: " << diffCN
                    << "\n    tolerance:  " << tol);
     }
 
     BarrierOption barrierOption(
         Barrier::DownOut, 85.0, 10.0,
-        payoff, boost::make_shared<EuropeanExercise>(maturity));
+        payoff, ext::make_shared<EuropeanExercise>(maturity));
 
     barrierOption.setPricingEngine(
-        boost::make_shared<FdHestonBarrierEngine>(model, 100, 31, 11));
+        ext::make_shared<FdHestonBarrierEngine>(model, 100, 31, 11));
 
     const Real expectedBarrier = barrierOption.NPV();
 
     barrierOption.setPricingEngine(
-        boost::make_shared<FdHestonBarrierEngine>(model, 100, 31, 11, 0,
+        ext::make_shared<FdHestonBarrierEngine>(model, 100, 31, 11, 0,
             FdmSchemeDesc::MethodOfLines()));
 
-    const Real calculatedBarrier = barrierOption.NPV();
+    const Real calculatedBarrierMoL = barrierOption.NPV();
 
     const Real barrierTol = 0.01;
-    const Real barrierDiff = std::fabs(expectedBarrier - calculatedBarrier);
+    const Real barrierDiffMoL = std::fabs(expectedBarrier - calculatedBarrierMoL);
 
-    if (barrierDiff > barrierTol) {
+    if (barrierDiffMoL > barrierTol) {
         BOOST_FAIL("Failed to reproduce barrier option values with MOL"
-                   << "\n    calculated: " << calculatedBarrier
+                   << "\n    calculated: " << calculatedBarrierMoL
                    << "\n    expected:   " << expectedBarrier
-                   << "\n    difference: " << barrierDiff
+                   << "\n    difference: " << barrierDiffMoL
+                   << "\n    tolerance:  " << barrierTol);
+    }
+
+    barrierOption.setPricingEngine(
+        ext::make_shared<FdHestonBarrierEngine>(model, 100, 31, 11, 0,
+            FdmSchemeDesc::CrankNicolson()));
+
+    const Real calculatedBarrierCN = barrierOption.NPV();
+    const Real barrierDiffCN = std::fabs(expectedBarrier - calculatedBarrierCN);
+
+    if (barrierDiffCN > barrierTol) {
+        BOOST_FAIL("Failed to reproduce barrier option values with Crank-Nicolson"
+                   << "\n    calculated: " << calculatedBarrierCN
+                   << "\n    expected:   " << expectedBarrier
+                   << "\n    difference: " << barrierDiffCN
                    << "\n    tolerance:  " << barrierTol);
     }
 }
 
+void FdHestonTest::testSpuriousOscillations() {
+    BOOST_TEST_MESSAGE("Testing for spurious oscillations when "
+            "solving the Heston PDEs...");
+
+    SavedSettings backup;
+
+    const DayCounter dc = Actual365Fixed();
+    const Date today = Date(7, June, 2018);
+
+    Settings::instance().evaluationDate() = today;
+
+    const Handle<Quote> spot(ext::make_shared<SimpleQuote>(100.0));
+    const Handle<YieldTermStructure> qTS(flatRate(today, 0.1, dc));
+    const Handle<YieldTermStructure> rTS(flatRate(today, 0.0, dc));
+
+    const Real v0    = 0.005;
+    const Real kappa = 1.0;
+    const Real theta = 0.005;
+    const Real sigma = 0.4;
+    const Real rho   = -0.75;
+
+    const Date maturity = today + Period(1, Years);
+
+    const ext::shared_ptr<HestonProcess> process =
+        ext::make_shared<HestonProcess>(
+            rTS, qTS, spot, v0, kappa, theta, sigma, rho);
+
+    const ext::shared_ptr<HestonModel> model =
+        ext::make_shared<HestonModel>(process);
+
+    const ext::shared_ptr<FdHestonVanillaEngine> hestonEngine(
+        ext::make_shared<FdHestonVanillaEngine>(
+            model, 6, 200, 13, 0, FdmSchemeDesc::TrBDF2()));
+
+    VanillaOption option(
+        ext::make_shared<PlainVanillaPayoff>(Option::Call, spot->value()),
+        ext::make_shared<EuropeanExercise>(maturity));
+
+    option.setupArguments(hestonEngine->getArguments());
+
+    const ext::tuple<FdmSchemeDesc, std::string, bool> descs[] = {
+        ext::make_tuple(FdmSchemeDesc::CraigSneyd(), "Craig-Sneyd", true),
+        ext::make_tuple(FdmSchemeDesc::Hundsdorfer(), "Hundsdorfer", true),
+        ext::make_tuple(
+           FdmSchemeDesc::ModifiedHundsdorfer(), "Mod. Hundsdorfer", true),
+        ext::make_tuple(FdmSchemeDesc::Douglas(), "Douglas", true),
+        ext::make_tuple(FdmSchemeDesc::CrankNicolson(), "Crank-Nicolson", true),
+        ext::make_tuple(FdmSchemeDesc::ImplicitEuler(), "Implicit", false),
+        ext::make_tuple(FdmSchemeDesc::TrBDF2(), "TR-BDF2", false)
+    };
+
+    for (const auto& desc : descs) {
+        const ext::shared_ptr<FdmHestonSolver> solver = ext::make_shared<FdmHestonSolver>(
+            Handle<HestonProcess>(process), hestonEngine->getSolverDesc(1.0), ext::get<0>(desc));
+
+        std::vector<Real> gammas;
+        for (Real x=99; x < 101.001; x+=0.1) {
+            gammas.push_back(solver->gammaAt(x, v0));
+        }
+
+        Real maximum = QL_MIN_REAL;
+        for (Size i=1; i < gammas.size(); ++i) {
+            const Real diff = std::fabs(gammas[i] - gammas[i-1]);
+            if (diff > maximum)
+                maximum = diff;
+        }
+
+        const Real tol = 0.01;
+        const bool hasSpuriousOscillations = maximum > tol;
+
+        if (hasSpuriousOscillations != ext::get<2>(desc)) {
+            BOOST_ERROR("unable to reproduce spurious oscillation behaviour "
+                        << "\n   scheme name          : " << ext::get<1>(desc)
+                        << "\n   oscillations observed: " << hasSpuriousOscillations
+                        << "\n   oscillations expected: " << ext::get<2>(desc));
+        }
+    }
+}
 
 test_suite* FdHestonTest::suite(SpeedLevel speed) {
-    test_suite* suite = BOOST_TEST_SUITE("Finite Difference Heston tests");
+    auto* suite = BOOST_TEST_SUITE("Finite Difference Heston tests");
 
     suite->add(QUANTLIB_TEST_CASE(&FdHestonTest::testFdmHestonVarianceMesher));
     suite->add(QUANTLIB_TEST_CASE(&FdHestonTest::testFdmHestonBarrier));
@@ -806,8 +990,8 @@ test_suite* FdHestonTest::suite(SpeedLevel speed) {
         &FdHestonTest::testFdmHestonEuropeanWithDividends));
     suite->add(QUANTLIB_TEST_CASE(
         &FdHestonTest::testFdmHestonIntradayPricing));
-    suite->add(QUANTLIB_TEST_CASE(
-        &FdHestonTest::testMethodOfLines));
+    suite->add(QUANTLIB_TEST_CASE(&FdHestonTest::testMethodOfLinesAndCN));
+    suite->add(QUANTLIB_TEST_CASE(&FdHestonTest::testSpuriousOscillations));
 
     if (speed <= Fast) {
         suite->add(QUANTLIB_TEST_CASE(
